@@ -1,9 +1,6 @@
 import numpy as np
 
 from .. import Layer, to_matrix
-from .utils import tensorflow_shutup
-
-# tensorflow_shutup()
 
 
 class ClassifierLayer(Layer):
@@ -234,7 +231,8 @@ class TFMLP(ClassifierLayer):
                 save_best_model = SaveBestModel()
             else:
                 save_best_model = SaveBestModel("val_accuracy", True)
-            self.model.fit(X, y, validation_data=(X_test, y_test),
+            self.model.fit(X, y,
+                           validation_data=(X_test, y_test),
                            callbacks=[save_best_model], **self.options)
             self.model.set_weights(save_best_model.best_weights)
         else:
@@ -366,13 +364,13 @@ class Keras(Layer):
                     self.config, custom_objects=custom_objects)
         except ValueError:
             # add standard layers to Keras model if there's a ValueError
-            from .layers import MultiACTS, SelfAttention, WeightedAttention
+            from .layers import SelfAttention, WeightedAttention
             if custom_objects is None:
                 custom_objects = {}
             custom_objects["WeightedAttention"] = WeightedAttention
             custom_objects["SelfAttention"] = SelfAttention
-            custom_objects["MultiACTS"] = MultiACTS
             try:
+                print(self.config)
                 model = KModel.from_config(
                     self.config, custom_objects=custom_objects)
             except KeyError:
@@ -475,14 +473,17 @@ class Transformer(Layer):
         del self.model
 
     def reload(self, **_):
+        from keras_multi_head import MultiHead
+        from keras_self_attention import SeqSelfAttention
         from tensorflow.keras.models import Model as KModel
         from tensorflow.keras.models import Sequential
 
-        from .layers import SelfAttention, WeightedAttention
+        from .layers import WeightedAttention
 
         custom_objects = {
-            "SelfAttention": SelfAttention,
-            "WeightedAttention": WeightedAttention
+            "WeightedAttention": WeightedAttention,
+            "SeqSelfAttention": SeqSelfAttention,
+            "MultiHead": MultiHead
         }
         try:
             model = KModel.from_config(
@@ -539,10 +540,11 @@ class Transformer(Layer):
         input_len, embedding_dim, n_features, n_classes
     ):
         from tensorflow.keras.layers import (Activation, BatchNormalization,
-                                             Dense, Embedding, InputLayer)
+                                             Dense, Embedding, Flatten,
+                                             InputLayer, Reshape)
         from tensorflow.keras.models import Sequential
 
-        from .layers import SelfAttention, WeightedAttention
+        from .layers import WeightedAttention
         model = Sequential()
         model.add(InputLayer(input_shape=(input_len,)))
 
@@ -562,11 +564,19 @@ class Transformer(Layer):
 
         # self attention layers
         for _ in range(self.n_encoders):
-            model.add(SelfAttention(self.encoder_dim,
-                      self.n_heads, self.l1,
-                      activation=self.encoder_activation))
+            from keras_multi_head import MultiHead
+            from keras_self_attention import SeqSelfAttention
+
+            model.add(MultiHead(
+                SeqSelfAttention(
+                    self.encoder_dim,
+                    attention_activation=self.encoder_activation,
+                    attention_type=SeqSelfAttention.ATTENTION_TYPE_MUL),
+                layer_num=self.n_heads,))
             model.add(BatchNormalization())
 
+        model.add(Reshape(
+            (input_len, embedding_dim * self.n_heads)))
         # weighted attention layer
         model.add(WeightedAttention(
             self.weighted_dim, self.n_weighted, self.l1,
