@@ -2,8 +2,9 @@ from .classifier import (MLP, RNN, SVM, TFMLP, AdaBoost, Keras, RandomForest,
                          Transformer, XGBoost)
 from .embedding import (CountVectorizer, Doc2Vec, OddsVectorizer, OneHot,
                         Sequence, TfIdf)
-from .linguistics import (NER, Contract, FindAll, LangDetect, Lemmatize, NGram,
-                          Phrase, Snowball, Sub, Tokenize)
+from .linguistics import (NER, BPETokenize, Contract, CountFilter, FindAll,
+                          LangDetect, Lemmatize, NGram, Phrase, Snowball, Sub,
+                          Tokenize)
 from .multi import Merge
 from .preprocessing import Normalize
 from .reduction import NMF, PCA, SVD, UMAP, RandomizedSVD
@@ -17,5 +18,6 @@ __all__ = [
     "RandomForest", "AdaBoost", "SVM", "MLP", "Keras", "Transformer",
     "XGBoost", "Merge", "Normalize", "SplitHashtag", "TFMLP",
     "MLPRegressor", "KerasRegressor", "Lambda", "RNN", "DomainName",
-    "RandomizedSVD", "NER", "Argmax", "OddsVectorizer"
+    "RandomizedSVD", "NER", "Argmax", "OddsVectorizer", "BPETokenize",
+    "CountFilter"
 ]
